from unittest import TestCase

from sister.word_embedders import WordEmbedding, FasttextEmbedding


class WordEmbeddingCase(TestCase):

    def setUp(self):
        self.words = ['I', 'am', 'a', 'dog', '.']

    def tearDown(self):
        pass

    def test_get_word_vector_not_implemented(self):
        class Dummy(WordEmbedding):
            def get_word_vectors(self, w): ...
        with self.assertRaises(NotImplementedError):
            words = self.words
            Dummy().get_word_vector(words)


class FasttextEmbeddingCase(TestCase):

    def setUp(self):
        # TODO
        # Downloading fasttext and unzipping is too heavy.
        # Needs to be mocked somehow.
        # For now, solved with caching.

        # embedding_patcher = patch('sister.word_embedders.FasttextEmbedding')
        # self.embedding = embedding_patcher.start()(lang='en')
        # self.embedding.get_word_vector.return_value = np.random.rand(300)
        # self.embedding.get_word_vectors.side_effect = lambda words: np.random.rand(len(words), 300)
        # self.embedding_patcher = embedding_patcher
        self.embedding = FasttextEmbedding(lang="en")

    def tearDown(self):
        # self.embedding_patcher.stop()
        pass

    def test_get_word_vector(self):
        word = 'test'
        vector = self.embedding.get_word_vector(word)
        self.assertTupleEqual(vector.shape, (300,))
        # self.embedding.get_word_vector.assert_called_once_with(word)

    def test_get_word_vectors(self):
        words = ['good', 'test']
        vectors = self.embedding.get_word_vectors(words)
        self.assertTupleEqual(vectors.shape, (len(words), 300))
        # self.embedding.get_word_vectors.assert_called_once_with(words)



from typing import List
from janome.tokenizer import Tokenizer as JanomeTokenizer


class Tokenizer(object):

    def tokenize(self, sentence: str) -> List[str]:
        raise NotImplementedError


class SimpleTokenizer(Tokenizer):

    def __init__(self):
        self.replace_tokens = str.maketrans({
            '.': ' .',
            '?': ' ?',
            '!': ' !',
            '(': ' ( ',
            ')': ' ) ',
        })

    def tokenize(self, sentence: str) -> List[str]:
        return sentence.translate(self.replace_tokens).split()


class JapaneseTokenizer(Tokenizer):

    def __init__(self):
        self._t = JanomeTokenizer()

    def tokenize(self, sentence: str) -> List[str]:
        return self._t.tokenize(sentence, wakati=True)



from unittest import TestCase
from unittest.mock import patch

import numpy as np

from sister.tokenizers import Tokenizer, SimpleTokenizer
from sister.word_embedders import WordEmbedding
from sister import MeanEmbedding
from sister.core import SentenceEmbedding


class SentenceEmbeddingCase(TestCase):

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_embed_not_implemented(self):
        class Dummy(SentenceEmbedding):
            def __call__(self, s): ...
        sentence = 'I am a dog.'
        with self.assertRaises(NotImplementedError):
            Dummy(Tokenizer(), WordEmbedding()).embed(sentence)

    def test_call_not_implemented(self):
        class Dummy(SentenceEmbedding):
            def embed(self, s): ...
        sentence = 'I am a dog.'
        with self.assertRaises(NotImplementedError):
            Dummy(Tokenizer(), WordEmbedding()).__call__(sentence)


class MeanEmbeddingCase(TestCase):

    def setUp(self):
        embedding_patcher = patch('sister.word_embedders.FasttextEmbedding')
        embedding = embedding_patcher.start()(lang='en')
        embedding.get_word_vector.return_value = np.random.rand(300)
        embedding.get_word_vectors.side_effect = lambda words: np.random.rand(len(words), 300)

        self.sentence_embedding = MeanEmbedding(
            tokenizer=SimpleTokenizer(),
            word_embedder=embedding
        )
        self.addCleanup(embedding_patcher.stop)

    def tearDown(self):
        pass

    def test_embed(self):
        sentence = "I am a dog."
        vector = self.sentence_embedding(sentence)
        self.assertEqual(vector.shape, (300,))
